﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

//found here http://answers.unity3d.com/questions/190800/wrapping-a-textmesh-text.html
// includes both TextSize.cs and modifications isted in the answers

/**
 
 TextSize for Unity3D by thienhaflash (thienhaflash@gmail.com)
 
 Version	: 0.1
 Update		: 18.Jun.2012
 Features	:
	Return perfect size for any TextMesh
 	Cache the size of each character to speed up the size
	Evaluate and cache only when there are requirements
 
 Sample 	:
		
		//declare it locally, so we can have access anywhere from the script
		TextSize ts;
		
		//put this on the Start function
	 	ts = new TextSize(gameObject.GetComponent<TextMesh>());
		
		//anywhere, after you change the text :
		print(ts.width);
		
		//or get the length of an abitrary text (that is not assign to TextMesh)
		print(ts.GetTextWidth("any abitrary text goes here"));

 You are free to use the code or modify it any way you want (even remove this block of comments) but if it's good
 please give it back to the community.
 
 */

public class TextSize {
	private Hashtable dict; //map character -> width
	
	private TextMesh textMesh;
	private Renderer renderer;
	
	public TextSize(TextMesh tm){
		textMesh = tm;
		renderer = tm.renderer;
		dict = new Hashtable();
		getSpace();
	}

	public static string UntagString(string s, bool colourOnly){
		string retString = "";
		bool inMarkup = false;
		for (int i=0; i<s.Length; i++){
			if (s.Substring(i,1) == "<") inMarkup = true;

			if (colourOnly){
				if (i<s.Length-2 && s.Substring(i,3) == "<b>") inMarkup = false;
				if (i<s.Length-3 && s.Substring(i,4) == "</b>") inMarkup = false;
				if (i<s.Length-2 && s.Substring(i,3) == "<u>") inMarkup = false;
				if (i<s.Length-3 && s.Substring(i,4) == "</u>") inMarkup = false;
				if (i<s.Length-2 && s.Substring(i,3) == "<i>") inMarkup = false;
				if (i<s.Length-3 && s.Substring(i,4) == "</i>") inMarkup = false;
			}

			if (!inMarkup) retString += s.Substring(i,1);

			
			if (s.Substring(i,1) == ">") inMarkup = false;
		}

		return retString;
	}

	public static string RetagString(string s, string tagVersion){
		string retString = "";//s;

		bool done = false;
		int sindex = 0;
		int tindex = 0;

		//if (s != tagVersion) Debug.Log("DIFFERENT!?");
		
		List<string> tags = new List<string>();
		List<bool> tagsClosed = new List<bool>();

		if (s.Length<1){
			return s;
		}

		while (!done){

			//done = true;


			if (s.Substring(sindex,1) != tagVersion.Substring(tindex,1)){

				if (s.Substring(sindex,1) == "\n"){

				}
				if (tagVersion.Substring(tindex,1) == "<"){
					//this is taggy stuff, add it until we are back on track
					string thisTag = "";
					bool endOfTag = false;
					bool ignoreTag = false;
					int thisTindex = tindex;
					while (!endOfTag){

						thisTag += tagVersion.Substring(thisTindex,1);
						if (tagVersion.Substring(thisTindex,1) == ">") endOfTag = true;
						thisTindex++;
						if (thisTindex >= tagVersion.Length){
							//EOF, ignore this incomplete tag
							ignoreTag = true;
							endOfTag = true;
						}
					}
					if (!ignoreTag){
						retString += thisTag;
						tags.Add(thisTag.Substring(1,thisTag.Length-2));
						tagsClosed.Add(false);

						if (tags[tags.Count-1].Substring(0,1) == "/"){
							//we're closing a tag, find out which
							tagsClosed[tagsClosed.Count-1] = true;

							bool closed = false;
							for (int i=0; i<tags.Count-1; i++){
								string subtagThis = tags[tags.Count-1].Substring(1);
								string subtagThat = tags[i].Split('=')[0];
								if (!tagsClosed[i] && !closed && subtagThis == subtagThat){
									tagsClosed[i] = true;
									closed = true;
								}
							}
						}
					}
					tindex += thisTag.Length;
				}else{
					//new line added when wrapping
					retString += s.Substring(sindex,1);
					sindex++;
					tindex++;
				}

			}else{
				retString += s.Substring(sindex,1);
				sindex++;
				tindex++;
			}

			if (tindex>=tagVersion.Length){
				done = true;
			}
			if (sindex>=s.Length){
				done = true;
			}


		}

		for (int i=tags.Count-1; i>=0; i--){
			if (!tagsClosed[i]){
				string closeTag = "</" + tags[i].Split('=')[0] + ">";
				retString += closeTag;
			}
			//Debug.Log(tags[i]);
		}



		return retString;
	}
	
	private void getSpace(){//the space can not be got alone
		string oldText = textMesh.text;
		
		textMesh.text = "a";
		float aw = renderer.bounds.size.x;
		textMesh.text = "a a";
		float cw = renderer.bounds.size.x - 2* aw;
		
		//MonoBehaviour.print("char< > " + cw);
		dict.Add(' ', cw);
		dict.Add('a', aw);
		
		textMesh.text = oldText;
	}


	
	public float GetTextWidth(string s) {
		char[] charList = s.ToCharArray();
		float w = 0;
		char c;
		string oldText = textMesh.text;
		
		for (int i=0; i<charList.Length; i++){
			c = charList[i];
			
			if (dict.ContainsKey(c)){
				w += (float)dict[c];
			} else {
				textMesh.text = ""+c;
				float cw = renderer.bounds.size.x;
				dict.Add(c, cw);
				w += cw;
				//MonoBehaviour.print("char<" + c +"> " + cw);
			}
		}
		
		textMesh.text = oldText;
		return w;
	}


	
	public float width { get { return GetTextWidth(textMesh.text); } }
	public float height { get { return renderer.bounds.size.y; } }

	// modifications sart

	public void FitToWidth(float wantedWidth) {
		
		if(width <= wantedWidth) return;
		
		string oldText = textMesh.text;
		textMesh.text = "";
		
		string[] lines = oldText.Split('\n');
		
		//foreach(string line in lines){
		//	textMesh.text += wrapLine(line, wantedWidth);
		//	textMesh.text += "\n";
		//}
		for(int i=0; i<lines.Length; i++){
			textMesh.text += wrapLine(lines[i], wantedWidth);
			if (i<lines.Length-1) textMesh.text += "\n";
		}
	}
	
	private string wrapLine(string s, float w)
	{
		// need to check if smaller than maximum character length, really...
		if(w == 0 || s.Length <= 0) return s;
		
		char c;
		char[] charList = s.ToCharArray();
		
		float charWidth = 0;
		float wordWidth = 0;
		float currentWidth = 0;
		
		string word = "";
		string newText = "";
		string oldText = textMesh.text;
		
		for (int i=0; i<charList.Length; i++){
			c = charList[i];
			
			if (dict.ContainsKey(c)){
				charWidth = (float)dict[c];
			} else {
				textMesh.text = ""+c;
				charWidth = renderer.bounds.size.x;
				dict.Add(c, charWidth);
				//here check if max char length
			}
			
			if(c == ' ' || i == charList.Length - 1){
				if(c != ' '){
					word += c.ToString();
					wordWidth += charWidth;
				}
				
				if(currentWidth + wordWidth < w){
					currentWidth += wordWidth;
					newText += word;
				} else {
					currentWidth = wordWidth;
					newText += word.Replace(" ", "\n");
				}
				
				word = "";
				wordWidth = 0;
			} 
			
			word += c.ToString();
			wordWidth += charWidth;
		}
		
		textMesh.text = oldText;
		return newText;
	}
}