﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class WordBubble : MonoBehaviour {


	public int letterCount = 10;

	[HideInInspector]
	public List<Letter> letters;

	public float letterRadius = 1f;

	public GameObject letterPrefab;

	[HideInInspector]
	public float camAngX = 0f;

	private string[] vowels = {"a","e","i","o","u"};


	void Start(){
		letterCount = Random.Range(5,20);

		camAngX = Random.Range(-40f,40f);
		SpawnLetters();
		transform.eulerAngles = new Vector3(Random.Range(-180f,180f),Random.Range(-180f,180f),Random.Range(-180f,180f));
	}

	// Use this for initialization
	void SpawnLetters() {
	
		letters = new List<Letter>();
		for (int i=0; i<letterCount; i++){
			GameObject newLetter = (GameObject)GameObject.Instantiate(letterPrefab);
			letters.Add(newLetter.GetComponent<Letter>());
			letters[letters.Count-1].Init("");
		}

		int vowelCount = 0;
		for (int i=0; i<letters.Count; i++){
		
			for (int k=0; k<vowels.Length; k++){
				if (letters[i].letter == vowels[k]) vowelCount++;
			}

		}
		int desiredVowelCount = letters.Count / 5;
		if (desiredVowelCount<=0) desiredVowelCount = 1;
		while (vowelCount < desiredVowelCount){

			letters[Random.Range(0,letters.Count)].Init(vowels[Random.Range(0,vowels.Length)]);
			vowelCount++;

		}

		PositionLetters();
	}

	public int TypeLetter(string typedLetter){

		for (int i=0; i<letters.Count; i++){
			if (letters[i].letter == typedLetter && !letters[i].used){
				letters[i].used = true;
				return i;
			}
		}

		return -1;
	}

	void PositionLetters(){
		for (int i=0; i<letterCount; i++) letters[i].collider.enabled = false;

		for (int i=0; i<letterCount; i++){
			letters[i].transform.parent = transform;
			letters[i].transform.position = transform.position;
			letters[i].transform.eulerAngles = new Vector3(Random.Range(-180f,180f),Random.Range(-180f,180f),Random.Range(-180f,180f));

			Vector3 origin = transform.position + (letters[i].transform.forward * 20f);
			
			letters[i].collider.enabled = false;
			RaycastHit hit = new RaycastHit();
			if (Physics.SphereCast(origin, letterRadius, -letters[i].transform.forward, out hit, 20f)){
				letters[i].transform.position = hit.point + (hit.normal*letterRadius);
			}
			if (i==0){
				letters[i].transform.position = transform.position;
				
			}
			letters[i].collider.enabled = true;

			
		}
	}

	float shakeAmount = 0f;
	public void Shake(){
		shakeAmount = 2f;
	}
	
	// Update is called once per frame
	void Update () {
		//if (Input.GetKeyDown("space")) PositionLetters ();
		//if (Input.GetKeyDown("space")) Shake ();

		shakeAmount -= Time.deltaTime * 2f;
		if (shakeAmount<0f) shakeAmount = 0f;

		/*
		// FUCKING ORBITS WOA!
		for (int i=0; i<letters.Count; i++){
			Vector3 dirToCenter = transform.position - letters[i].transform.position;
			letters[i].forces += dirToCenter.normalized * Time.deltaTime * 5f;

			for (int k=0; k<letters.Count; k++){
				Vector3 offset = letters[k].transform.position - letters[i].transform.position;
				float pushStrength = (letterRadius - offset.magnitude) *-1f;
				if (pushStrength<0f) pushStrength = 0f;
				letters[i].forces += offset.normalized * Time.deltaTime * pushStrength;
			}

			letters[i].transform.position += letters[i].forces * Time.deltaTime;
		}*/

		for (int i=0; i<letters.Count; i++){
			Vector3 dirToCenter = transform.position - letters[i].transform.position;
			letters[i].forces += dirToCenter.normalized * Time.deltaTime * 10f;

			for (int k=0; k<letters.Count; k++){
				Vector3 offset = letters[k].transform.position - letters[i].transform.position;
				if (offset.magnitude<letterRadius){
					Vector3 centerPos = letters[i].transform.position + (offset*0.5f);
					letters[i].transform.position = centerPos - (offset.normalized*letterRadius*0.5f);
					letters[k].transform.position = centerPos + (offset.normalized*letterRadius*0.5f);
				}
			}
			for (int k=0; k<letters.Count; k++){
				Vector3 offset = letters[k].transform.position - letters[i].transform.position;
				float pushStrength = Mathf.Clamp(offset.magnitude,0f,letterRadius*(pushRad+shakeAmount));
				if (pushStrength>0f) pushStrength = pushStrength/(letterRadius*(pushRad+shakeAmount));
				pushStrength = 1f-pushStrength;
				letters[i].forces -= offset.normalized * Time.deltaTime * pushStrength * 8f;
			}

			letters[i].forces += shakeAmount * new Vector3(Random.Range(-1f,1f),Random.Range(-1f,1f),Random.Range(-1f,1f)) * 0.5f;

			letters[i].forces = Vector3.Lerp(letters[i].forces, Vector3.zero, Time.deltaTime);
			letters[i].transform.position += letters[i].forces * Time.deltaTime;
		}

		transform.Rotate(0f, Time.deltaTime * 10f, 0f);

	}
	public float pushRad = 2f;
}
