﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class WordlScript : MonoBehaviour {

	public List<WordBubble> bubbles;
	private int bubbleIndex = 0;

	public Transform cam;
	private Vector3 camRot = Vector3.zero;

	public TextMesh outputText;

	private string[] alphabet = {"a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"};

	private TextSize ts;

	// Use this for initialization
	void Start () {
		ts = new TextSize(outputText);
	}

	private float cursorFlash = 0f;
	
	// Update is called once per frame
	void Update () {

		if (Input.GetKeyDown("escape")) Application.LoadLevel("asdf");
		
		if (Input.GetKeyDown("space")) AddLetter(bubbleIndex, -1);

		for (int i=0; i<alphabet.Length; i++){
			if (Input.GetKeyDown(alphabet[i])){

				int letterIndex = bubbles[bubbleIndex].TypeLetter(alphabet[i]);
				if (letterIndex == -1){
					//letter not found here, shake the word bubble
					bubbles[bubbleIndex].Shake();
				}else{
					//letter found and claimed
					AddLetter(bubbleIndex, letterIndex);
				}

			}
		}

		
		if (Input.GetKeyDown("backspace")) RemoveLetter();


		string outputString = "";
		for (int i=0; i<characters.Count; i++){
			outputString += characters[i].chara;
		}
		cursorFlash-= Time.deltaTime * 1.5f;
		if (cursorFlash<-1f) cursorFlash += 2f;
		if (cursorFlash<0f){
			outputString += "_";
		}else{
			outputString += " ";
		}
		outputText.text = outputString;
		ts.FitToWidth(22f);

		PositionLine();

		Vector3 camPosFrom = cam.position;
		Quaternion camRotFrom = cam.rotation;

		cam.position = bubbles[bubbleIndex].transform.position;
		camRot.y += Time.deltaTime * 20f;
		camRot.x = bubbles[bubbleIndex].camAngX;
		cam.eulerAngles = camRot;
		cam.position -= cam.forward * 10f;

		cam.position = Vector3.Lerp(camPosFrom, cam.position, Time.deltaTime * 10f);
		cam.rotation = Quaternion.Slerp(camRotFrom, cam.rotation, Time.deltaTime * 10f);
	}

	void PositionLine(){
		List<Vector3> points = new List<Vector3>();

		for (int i=0; i<characters.Count; i++){
			if (characters[i].letterIndex != -1){
				Letter letter = bubbles[characters[i].bubbleIndex].letters[characters[i].letterIndex];
				points.Add(letter.transform.position + (letter.transform.forward*0.5f));
			}
		}

		//tidy that line up
		List<Vector3> pointsB = new List<Vector3>();
		for (int i=0; i<points.Count-1; i++){
			Vector3 p1 = points[i];
			Vector3 p2 = points[i+1];

			Vector3 dir = (p2-p1).normalized * 0.1f;
			pointsB.Add(p1);
			pointsB.Add(p1 + dir);
			pointsB.Add(p2 - dir);
		}


		lr.SetVertexCount(pointsB.Count);
		for (int i=0; i<pointsB.Count;i++){
			lr.SetPosition(i,pointsB[i]);
		}
	}

	public GameObject bubblePrefab;


	public LineRenderer lr;

	private List<EnteredChara> characters = new List<EnteredChara>();
	void AddLetter(int bIndex, int lIndex){

		if (lIndex == -1 && bubbleIndex>=bubbles.Count-1){
			//add new bubble
			GameObject newBubble = (GameObject)GameObject.Instantiate(bubblePrefab);
			newBubble.transform.position = bubbles[bubbleIndex].transform.position + new Vector3(Random.Range(15f,30f),Random.Range(5f,-1f),Random.Range(20f,-20f));
			bubbles.Add(newBubble.GetComponent<WordBubble>());
			//return;//no spaces after the last bubble
		}
		if (lIndex == -1){
			if (characters.Count>0){
				if (characters[characters.Count-1].letterIndex==-1){
					//don't allow two spaces in a row, no bubble skipping allowed!
					bubbles[bubbleIndex].Shake();
					return;
				}
			}else{
				//no space at the start, skipping first bubble is banned!
				bubbles[bubbleIndex].Shake();
				return;
			}
		}

		EnteredChara newChara = new EnteredChara();
		if (lIndex != -1) newChara.chara = bubbles[bIndex].letters[lIndex].letter.ToUpper();
		if (lIndex == -1) newChara.chara = " ";
		newChara.letterIndex = lIndex;
		newChara.bubbleIndex = bIndex;
		characters.Add(newChara);

		if (lIndex == -1) bubbleIndex++;
	}
	void RemoveLetter(){
		if (characters.Count == 0){
			bubbles[bubbleIndex].Shake();
			return;
		}
		if (characters[characters.Count-1].letterIndex!=-1){
			//remove letter
			bubbles[characters[characters.Count-1].bubbleIndex].letters[characters[characters.Count-1].letterIndex].used = false;
		}else{
			//remove space
			bubbleIndex--;
		}
		characters.RemoveAt(characters.Count-1);
	}
}

public class EnteredChara{
	public string chara = "";
	public int letterIndex = 0;
	public int bubbleIndex = 0;

}